<?php
/*
Plugin Name: Custom Posts Per Page
Plugin URI: http://www.jeremyfelt.com/wordpress/plugins/custom-posts-per-page/
Description: Shows a custom set number of posts depending on the page.
Version: 1.0
Author: Jeremy Felt
Author URI: http://www.jeremyfelt.com
License: GPL2
*/

/*  Copyright 2011 Jeremy Felt (email: jeremy.felt@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

register_activation_hook( __FILE__, 'cpppc_activate' );

if ( is_admin() ){
	/*	If we're on the admin screen, we'll want to make sure that
		the appropriate settings are showing up. If we add the modified
		query here, the posts view in the admin screen changes as well. */
    add_action( 'admin_menu', 'cpppc_add_settings' );
	add_action( 'admin_init', 'cpppc_register_settings' );
}else { 
	/*	If we're on any page other than the admin screen, we'll add our 
		request modification. */
	add_filter( 'request', 'cpppc_modify_query' );
}

function cpppc_activate() {
	/*	When the plugin is first activated, we'll set some default values in
		an options array. We'll pull the default value from the current Reading setting
		for 'posts_per_page' so that nothing changes unexpectedly. */
	$default_count = get_option( 'posts_per_page' );
	
	/*	When the plugin is first activated, we'll create the options array and
		assign a default value of 10 to each of the other options. */
	$cpppc_options[ 'index_count' ] = $default_count;
	$cpppc_options[ 'category_count' ] = $default_count;
	$cpppc_options[ 'tag_count' ] = $default_count;
	$cpppc_options[ 'author_count' ] = $default_count;
	$cpppc_options[ 'archive_count' ] = $default_count;
	$cpppc_options[ 'search_count' ] = $default_count;
	$cpppc_options[ 'default_count' ] = $default_count;

	/*	We'll also get all of the currently registered custom post types
		and give them a default value of 0. */
	$all_post_types = get_post_types( array( '_builtin' => false ) );
	foreach ( $all_post_types as $p=>$k ){
		$cpppc_options[ $p . '_count' ] = 0;
	}
	
	add_option( 'cpppc_options', $cpppc_options );
}

function cpppc_add_settings() {
	/*	Add the sub-menu item under the Settings top-level menu. */
	add_options_page( __('Posts Per Page', 'post-counts' ), __('Posts Per Page', 'post-counts'), 'manage_options', 'post-count-settings', 'cpppc_view_settings' );
}

function cpppc_view_settings() {
	/*	Display the main settings view for Custom Posts Per Page. */
	echo '<div class="wrap">
		<div class="icon32" id="icon-options-general"></div>
			<h2>' . __( 'Custom Posts Per Page', 'post-counts' ) . '</h2>
			<h3>Overview:</h3>
			<p style="margin-left:12px;max-width:640px;">The settings below allow you to specify how many posts per page are displayed to readers depending on the which type of page is being viewed. Different values can be set for your home page (index), category pages, tag pages, author pages, archive pages, search pages and custom post type pages. In addition to these, a default value is available that can be set for any other pages not covered by this. The initial value has been pulled from <i>Blog Pages show at most</i> setting under <i>Reading</i>.</p>
			<form method="post" action="options.php">';
	
	settings_fields( 'cpppc_options' );
	do_settings_sections( 'cpppc' ); // Display the main section of settings.
	do_settings_sections( 'cpppc_custom' ); // Display the section of settings that handles custom post types.
	
	echo '<p class="submit"><input type="submit" class="button-primary" value="';
	_e('Save Changes');
	echo '" />
			</p>
			</form>
		</div>';
	
}

function cpppc_register_settings() {
	/*	Add the settings that we're going to be using for the plugin. */
	register_setting( 'cpppc_options', 'cpppc_options', 'cpppc_options_validate' );
	add_settings_section( 'cpppc_section_main', '', 'cpppc_section_text', 'cpppc' );
	add_settings_section( 'cpppc_section_custom', '', 'cpppc_section_custom_text', 'cpppc_custom' );
	add_settings_field( 'cpppc_index_count', 'Index posts per page:', 'cpppc_index_count_text', 'cpppc', 'cpppc_section_main' );
	add_settings_field( 'cpppc_category_count', 'Category posts per page:', 'cpppc_category_count_text', 'cpppc', 'cpppc_section_main' );
	add_settings_field( 'cpppc_archive_count', 'Archive posts per page:', 'cpppc_archive_count_text', 'cpppc', 'cpppc_section_main' );
	add_settings_field( 'cpppc_tag_count', 'Tag posts per page:', 'cpppc_tag_count_text', 'cpppc', 'cpppc_section_main' );
	add_settings_field( 'cpppc_author_count', 'Author posts per page:', 'cpppc_author_count_text', 'cpppc', 'cpppc_section_main' );
	add_settings_field( 'cpppc_search_count', 'Search posts per page:', 'cpppc_search_count_text', 'cpppc', 'cpppc_section_main' );
	add_settings_field( 'cpppc_default_count', 'Default posts per page:', 'cpppc_default_count_text', 'cpppc', 'cpppc_section_main' );
	add_settings_field( 'cpppc_post_type_count', '', 'cpppc_post_type_count_text', 'cpppc_custom', 'cpppc_section_custom' );
}

function cpppc_options_validate( $input ) {
	/*	We aren't doing heavy validation yet, more like a passive agressive failure.
		If you enter anything other than an integer, the value will be set to 0 by 
		default and if a negative value is inputted, it will be corrected to positive. */
	$input[ "index_count" ]    = absint( intval( $input[ "index_count" ] ) );
	$input[ "category_count" ] = absint( intval( $input[ "category_count" ] ) );
	$input[ "archive_count" ]  = absint( intval( $input[ "archive_count" ] ) );
	$input[ "tag_count" ]      = absint( intval( $input[ "tag_count" ] ) );
	$input[ "author_count" ]   = absint( intval( $input[ "author_count" ] ) );
	$input[ "search_count" ]   = absint( intval( $input[ "search_count" ] ) );
	$input[ "default_count" ]  = absint( intval( $input[ "default_count" ] ) );
	
	$all_post_types = get_post_types( array( '_builtin' => false ) );
	foreach ( $all_post_types as $p=>$k ) {
		$input[ $p . '_count' ] = absint( intval( $input[ $p . '_count' ] ) );
	}
	
	return $input;
}

function cpppc_section_text() {
	echo '<h3>Main Settings:</h3>
		<p style="max-width:640px;margin-left:12px;">This section allows you to modify page view types that are associated with WordPress by default. When an option is set to 0, it will not modify any page requests for that view and will instead allow default values to pass through.</p>';
}

function cpppc_section_custom_text() {
	echo '<h3>Custom Post Type Specific Settings:</h3>
	<p style="max-width:640px;margin-left:12px;">This section contains a list of all of your registerd custom post types. In order to not conflict with other plugins or themes, these are set to 0 by default. When an option is set to 0, it will not modify any page requests for that custom post type archive. For Custom Posts Per Page to control the number of posts to display, these will need to be changed.</p>';
}

function cpppc_post_type_count_text() {
	$cpppc_options = get_option( 'cpppc_options' );
	$all_post_types = get_post_types( array( '_builtin' => false ) );

	echo '</td><td></td></tr>';
	
	foreach( $all_post_types as $p=>$k ) {
		/*	Default values are assigned for custom post types that are available
			to us when our plugin is registered. If a custom post type becomes
			available after our plugin is installed, we'll want to catch it and
			assign a good value. */
		if ( ! isset( $cpppc_options[ $p . '_count' ] ) ){
			$cpppc_options[ $p . '_count' ] = 0;
		}
		
		$this_post_data = get_post_type_object($p);
		
		echo '<tr><td>';
		echo $this_post_data->labels->name . '</td><td> <input id="cpppc_post_type_count[' . $p . ']" name="cpppc_options[' . $p . '_count]" size="10" type="text" value="';
		echo $cpppc_options[ $p . '_count' ];
		echo '"></td></tr>';
	}
}

function cpppc_index_count_text() {
	/*	Display the input field for the index page post count option. */
	$cpppc_options = get_option( 'cpppc_options' );
	
	echo '<input id="cpppc_index_count" name="cpppc_options[index_count]" size="10" type="text" value="';
	echo $cpppc_options[ 'index_count' ];
	echo '" />';
}

function cpppc_category_count_text() {
	/*	Display the input field for the category page post count option. */
	$cpppc_options = get_option( 'cpppc_options' );
	
	echo '<input id="cppppc_category_count" name="cpppc_options[category_count]" size="10" type="text" value="';
	echo $cpppc_options[ 'category_count' ];
	echo '" />';
}

function cpppc_archive_count_text() {
	/*	Display the input field for the archive page post count option. */
	$cpppc_options = get_option( 'cpppc_options' );
	
	echo '<input id="cppppc_archive_count" name="cpppc_options[archive_count]" size="10" type="text" value="';
    echo $cpppc_options[ 'archive_count' ];
	echo '" />';
}

function cpppc_tag_count_text() {
	/*	Display the input field for the tag page post count option. */
	$cpppc_options = get_option( 'cpppc_options' );
	
	echo '<input id="cpppc_tag_count" name="cpppc_options[tag_count]" size="10" type="text" value="';
	echo $cpppc_options[ 'tag_count' ];
	echo '" />';
}

function cpppc_author_count_text() {
	/*	Display the input field for the author page post count option. */
	$cpppc_options = get_option( 'cpppc_options' );
	
	echo '<input id="cpppc_author_count" name="cpppc_options[author_count]" size="10" type="text" value="';
	echo $cpppc_options[ 'author_count' ];
	echo '" />';
}

function cpppc_search_count_text() {
	/*	Display the input field for the search page post count option. */
	$cpppc_options = get_option( 'cpppc_options' );
	
	echo '<input id="cppppc_search_count" name="cpppc_options[search_count]" size="10" type="text" value="';
	echo $cpppc_options[ 'search_count' ];
    echo '" />';
}

function cpppc_default_count_text() {
	/*	Display the input field for the default page post count option. */
	$cpppc_options = get_option( 'cpppc_options' );
	
	echo '<input id="cppppc_default_count" name="cpppc_options[default_count]" size="10" type="text" value="';
	echo $cpppc_options[ 'default_count' ];
    echo '" />';
}

function cpppc_modify_query( $request ) {
	/*	This is the important part of the plugin that actually modifies the query
		at the beginning of the page before anything is displayed. */
	$cpppc_options = get_option( 'cpppc_options' );
	$all_post_types = get_post_types( array( '_builtin' => false ) );
	$post_type_array = array();
	foreach ( $all_post_types as $p=>$k ) {
		$post_type_array[] = $p;
	}
	
	$cpppc_query = new WP_Query();
	$cpppc_query->parse_query( $request );
	
	if ( $cpppc_query->is_home() ) {
		if ( 0 != $cpppc_options[ 'index_count' ] ){
			$request[ 'posts_per_page' ] = $cpppc_options[ 'index_count' ];
		}
	}elseif( $cpppc_query->is_post_type_archive( $post_type_array ) ) {
		/*	We've just established that the visitor is loading an archive
			page of a custom post type by matching it to a general array.
			Now we'll loop back through until we find exactly what post type
			is matching so we can modify the request accordingly. */
		foreach( $post_type_array as $my_post_type ) {
			if( $cpppc_query->is_post_type_archive( $my_post_type ) ) {
				/*	Now we know for sure what custom post type we're on. */
				$my_post_type_option = $my_post_type;
			}
		}
		/*	Now check to see if we've assigned a value to this yet. When our
			plugin is registerd, only the custom post types available to us at
			the time are assigned options. If a new custom post type has been
			installed, it's possible it does not yet have an option. For now
			we'll skip the request modification and let it slide by if there is
			no match. */
		if( 0 != $cpppc_options[ $my_post_type_option . '_count' ] && isset( $cpppc_options[ $my_post_type_option . '_count' ] ) ){
			$request[ 'posts_per_page' ] = $cpppc_options[ $my_post_type_option . '_count' ];
		}
	}elseif ( $cpppc_query->is_category() ) {
		if ( 0 != $cpppc_options[ 'category_count' ] ){
			$request[ 'posts_per_page' ] = $cpppc_options[ 'category_count' ];
		}
	}elseif ( $cpppc_query->is_tag() ) {
		if ( 0 != $cpppc_options[ 'tag_count' ] ){
			$request[ 'posts_per_page' ] = $cpppc_options[ 'tag_count' ];
		}
	}elseif ( $cpppc_query->is_author() ) {
		if ( 0 != $cpppc_options[ 'author_count' ] ) {
			$request[ 'posts_per_page' ] = $cpppc_options[ 'author_count' ];
		}
	}elseif ( $cpppc_query->is_search() ) {
		if ( 0 != $cpppc_options[ 'search_count' ] ) {
			$request[ 'posts_per_page' ] = $cpppc_options[ 'search_count' ];
		}
	}elseif ( $cpppc_query->is_archive() ) {
		/*	Note that the check for is_archive needs to be below anything else
			that WordPress may consider an archive. This includes is_tag, is_category, is_author
			and probably some others.
		*/
		if ( 0 != $cpppc_options[ 'archive_count' ] ) {
			$request[ 'posts_per_page' ] = $cpppc_options[ 'archive_count' ];
		}
	}else{
		if ( 0 != $cpppc_options[ 'default_count' ] ) {
			$request[ 'posts_per_page' ] = $cpppc_options[ 'default_count' ];
		}
	}

	return $request;
}
	
?>